@@ -14,7 +14,7 @@ from .common import *
 import requests
 from itertools import cycle
 
-__version__ = '3.4.0'
+__version__ = '3.5.0'
 
 if NUMPY_INSTALLED:
     from .points_serializers import np_array
@@ -337,6 +337,8 @@ class Connection(object):
 
     def create_uploader_session(self):
         """
+        DEPRECATED: use Connection.upload_file() instead.
+
         Registers an uploader session.
 
         Used for uploading files to Shooju.
@@ -344,8 +346,7 @@ class Connection(object):
         :return: a UploaderSession instance
         :rtype: shooju.UploaderSession
         """
-        data = self.shooju_api.create_uploader_session()
-        return UploaderSession(self, data['session_id'])
+        return UploaderSession(self)
 
     def mget(self):
         """
@@ -382,7 +383,8 @@ class Connection(object):
     def _extract_series_tz(self, ser):
         return ser.get('tz', {}).get('timezone')
 
-    def get_df(self, query, fields, series_axis='rows', df=None, dt=None, max_points=0, sort=None, max_series=None, extra_params=None, scroll_batch_size=2500):
+    def get_df(self, query, fields, series_axis='rows', df=None, dt=None, max_points=0, sort=None, max_series=None,
+               extra_params=None, scroll_batch_size=2500):
         """
         Returns Pandas DataFrame; uses scroll() under the hood. All date indexes returned are tz-aware, to strip use Connection.get_df(.....).tz_localize(None)
         :param query: query that returns 1+ series; required non-empty
@@ -406,7 +408,7 @@ class Connection(object):
 
         response = list(self.scroll(serializer=pd_series_localized, **scroll_args))
         if series_axis == 'columns':
-            base_data = defaultdict(lambda: pandas.Series(None, dtype='float64'))
+            base_data = defaultdict(lambda: pandas.Series([], index=pandas.DatetimeIndex([], tz='UTC'), dtype='float64'))
             for series in response:
                 column_header = series['series_id']
                 if series.get('fields'):
@@ -426,7 +428,7 @@ class Connection(object):
                 series_by_dates = defaultdict(dict)
                 for series in response:
                     fields_by_sid[series['series_id']] = series.get('fields', dict())
-                    if series.get('points').empty:
+                    if 'points' not in series.keys() or series['points'].empty:
                         continue
                     for _date, _point in series['points'].items():
                         series_by_dates[_date][series['series_id']] = _point
@@ -443,6 +445,33 @@ class Connection(object):
 
         return pandas.DataFrame(data=base_data)
 
+    def upload_file(self, fp, filename):
+        """
+        Uploads a file object to Shooju. Returns the Shooju file id.
+
+        :param fp: File Object to upload
+        :param filename: Name of the file.  For cosmetic/retrieval purposes only.
+        """
+        data = self.raw.post(
+            '/files',
+            files={'file': (filename, fp)}
+        )
+        return data['file_id'][0]
+
+    def init_multipart_upload(self, filename):
+        """
+        Initiates multipart upload to upload big files by its parts.
+
+        :param filename: Name of the file.
+        :return: MultiPartUploader
+        """
+        data = self.raw.post(
+            '/files/multipart/init',
+            data_json={'filename': filename}
+        )
+
+        return MultiPartUploader(self, data['file_id'])
+
 
 def _parse_dt(d, param_name, default):
     if d is not None and not isinstance(d, six.string_types):
@@ -466,12 +495,12 @@ class ShoojuApi(object):
     API_SERIES_WRITE = '/series/write'
     API_SERIES_DELETE = '/series/delete'
     API_REPORTED_DATES = '/series/reported_dates'
+    API_REPORTED_DATES_DELETE = '/series/reported_dates/delete'
     API_JOBS = '/jobs/'
     API_FILES = '/files/{id}/download'
     API_STATUS = '/status/green'
     API_PING = '/status/ping'
     API_GOOGLE_OAUTH = '/auth/googleoauth'
-    API_UPLOADER_SESSION = '/uploader/session'
 
     def __init__(self, server, *args, **kwargs):
         server = server if server.startswith('http') else 'https://{}.shooju.com'.format(server)
@@ -495,14 +524,14 @@ class ShoojuApi(object):
         if not all((user, api_key)):
             raise ShoojuApiError('Must use either user+api_key or email+google_oauth_token to authenticate.')
 
-        self.extra_params = kwargs.get('extra_params', {})
         self.client = Client(server, user, api_key,
                              retries=kwargs.get('retries'),
                              retry_delay=kwargs.get('retry_delay'),
                              location=kwargs.get('location'),
                              hosts=kwargs.get('hosts'),
                              requests_session=requests_session,
-                             timeout=kwargs.get('timeout'))
+                             timeout=kwargs.get('timeout'),
+                             extra_params=kwargs.get('extra_params', {}))
 
     def get_single_series(self, series_id=None, series_query=None, df=None, dt=None,
                           max_points=None, fields=None, deserialize_response_to_numpy=None,
@@ -562,7 +591,6 @@ class ShoojuApi(object):
         if max_points is not None:
             url_params['max_points'] = max_points
 
-        url_params.update(self.extra_params)
         if extra_params:
             url_params.update(extra_params)
 
@@ -657,13 +685,6 @@ class ShoojuApi(object):
         return self.client.delete(self.API_SERIES_DELETE,
                                   data_json={'query': query, 'force': force, 'job_id': job_id})['deleted']
 
-    def create_uploader_session(self):
-        """
-        Registers uploader session.
-        :return:
-        """
-        return self.client.post(self.API_UPLOADER_SESSION)
-
     def register_job(self, description, notes='', source='python'):
         """
         Registers a job in Shooju.
@@ -805,6 +826,25 @@ class ShoojuApi(object):
             params['dt'] = to_milli(dt)
         return self.client.get(self.API_REPORTED_DATES, params=params)
 
+    def delete_reported(self, series_query, reported_dates=None, delete_all=False):
+        """
+        Deletes reported dates.
+
+        :param series_query: query string
+        :param reported_dates: `[default: empty]` delete only specific reported dates. If not provided will delete all.
+        :param delete_all: must be set to True to remove all reported dates when reported_dates is empty.
+        """
+        payload = {
+            'series_query': series_query,
+            'all': delete_all,
+        }
+        if reported_dates:
+            payload.update({
+                'date_format': 'milli',
+                'reported_dates': [to_milli(dt) for dt in reported_dates],
+            })
+        return self.client.post(self.API_REPORTED_DATES_DELETE, data_json=payload)
+
     def scroll_series(self, scroll_id):
         """
         Series scroll endpoint. Retrieves the next scroll data. Should be used in tandem with
@@ -844,7 +884,7 @@ class Client(object):
 
     def __init__(self, server, user=None, password=None, base_path='/api/1',
                  retries=None, retry_delay=None, location=None, hosts=None,
-                 requests_session=None, timeout=None):
+                 requests_session=None, timeout=None, extra_params=None):
         """
         REST Client
 
@@ -857,6 +897,7 @@ class Client(object):
         :param hosts: Array of Shooju API endpoints. Used when access by ip, or different host name.
         :param requests_session: session object of requests.Session type to use as transport
         :param timeout: requests timeout
+        :param extra_params: default url parameters that being sent to all api calls
         """
         if not hosts:
             hosts = [server]
@@ -873,6 +914,7 @@ class Client(object):
         self._methods = dict()
         self._session = requests_session
         self._timeout = timeout or DEFAULT_REQUEST_TIMEOUT
+        self._extra_params = extra_params or dict()
         for m in self.ALLOWED_METHODS:
             self._methods[m] = functools.partial(self._call, m)
             # copy these attributes to make it look like original function (fix functool.wraps  + sj.raw.get issues)
@@ -929,16 +971,16 @@ class Client(object):
         method = getattr(self._session if self._session is not None else requests, method_name)
 
         url = '{base}{path}'.format(base=next(self._endpoints), path=path)
-        params = params or {}
-
-        params.update({
+        url_params = dict(self._extra_params or {})
+        url_params.update(params or dict())
+        url_params.update({
             'v': __version__,
             'location_type': LOCATION_TYPE,
             'location': self._location
         })
         transient_failure = False
         try:
-            r = method(url, params=params or None, data=payload, headers=headers,
+            r = method(url, params=url_params or None, data=payload, headers=headers,
                        files=files, auth=self._auth, timeout=timeout)
             if r.status_code != requests.codes.ok:
                 transient_failure = r.status_code in [502, 503, 504]
@@ -1297,6 +1339,16 @@ class BaseJob(object):
         """
         pass
 
+    def delete_reported(self, series_query, reported_dates=None, delete_all=False):
+        """
+        Deletes reported dates.
+
+        :param series_query: query string
+        :param reported_dates: `[default: empty]` delete only specific reported dates. If not provided will delete all.
+        :param delete_all: must be set to True to remove all reported dates when reported_dates is empty.
+        """
+        pass
+
     def _add_hooks(self, hook_list, fn):
         if not hasattr(fn, '__call__'):
             raise ValueError('hooks must be a function')
@@ -1375,10 +1427,10 @@ class RemoteJob(BaseJob):
         for (query, reported_date), values in self._queries.items():
             # here we will construct a BULK API call
             if values['op'] == 'delete':
-                tmp_dict = {"type": "DELETE", "series_query": query, 'force': values['force']}
+                tmp_dict = {"type": "DELETE", "query": query, 'force': values['force']}
             else:  # post
                 points = list(sorted(six.iteritems(values['points'])))
-                tmp_dict = {"type": "POST", "series_query": query}
+                tmp_dict = {"type": "POST", "query": query}
                 if points:
                     tmp_dict['points'] = points
                 if values['fields']:
@@ -1454,6 +1506,16 @@ class RemoteJob(BaseJob):
         self.submit()
         return cnt + 1
 
+    def delete_reported(self, series_query, reported_dates=None, delete_all=False):
+        """
+        Deletes reported dates.
+
+        :param series_query: query string
+        :param reported_dates: `[default: empty]` delete only specific reported dates. If not provided will delete all.
+        :param delete_all: must be set to True to remove all reported dates when reported_dates is empty.
+        """
+        self._conn.shooju_api.delete_reported(series_query, reported_dates, delete_all)
+
 
 class DryRunJob(BaseJob):
     POINTS_PRINT_LIMIT = 5
@@ -1491,20 +1553,19 @@ class DryRunJob(BaseJob):
 
 class UploaderSession(object):
     """
+    DEPRECATED
     Used to upload files to Shooju via a session id.
 
     Do not instantiate directly.  Use conn.create_uploader_session().
     """
 
-    def __init__(self, conn, session_id):
+    def __init__(self, conn):
         """
         Initialized with connection and session_id.
 
         :param conn. Connection conn: API connection
-        :param session_id: uploader session id
         """
         self._conn = conn
-        self._session_id = session_id
 
     def upload_file(self, fp, filename):
         """
@@ -1513,11 +1574,7 @@ class UploaderSession(object):
         :param fp: File Object to upload
         :param filename: Name of the file.  For cosmetic/retrieval purposes only.
         """
-        data = self._conn.raw.post(
-            '/uploader/session/{}/files'.format(self._session_id),
-            files={'file': (filename, fp)}
-        )
-        return data['file_id'][0]
+        return self._conn.upload_file(fp, filename)
 
     def init_multipart(self, filename):
         """
@@ -1525,12 +1582,8 @@ class UploaderSession(object):
 
         :param filename: Name of the file.
         """
-        data = self._conn.raw.post(
-            '/uploader/session/{}/multipart/init'.format(self._session_id),
-            data_json={'filename': filename}
-        )
-
-        return data['file_id']
+        mp = self._conn.init_multipart_upload(filename)
+        return mp.file_id
 
     def upload_part(self, file_id, part_num, data):
         """
@@ -1540,12 +1593,8 @@ class UploaderSession(object):
         :param part_num: integer matching part number.
         :param data: File object or a data chunk to upload
         """
-        self._conn.raw.post(
-            '/uploader/session/{}/multipart/upload'.format(self._session_id),
-            params=dict(file_id=file_id, part_num=part_num), files=dict(file=data)
-        )
-
-        return file_id
+        mp = MultiPartUploader(self._conn, file_id)
+        return mp.upload_part(part_num, data)
 
     def complete_multipart(self, file_id):
         """
@@ -1553,12 +1602,45 @@ class UploaderSession(object):
 
         :param file_id: ID assigned to the file previously.
         """
-        data = self._conn.raw.post(
-            '/uploader/session/{}/multipart/complete'.format(self._session_id),
-            params=dict(file_id=file_id)
+        mp = MultiPartUploader(self._conn, file_id)
+        return mp.complete()
+
+
+class MultiPartUploader(object):
+    """
+    Handles multi-part files upload
+    """
+    def __init__(self, con, file_id):
+        """
+        Instantiates new uploader.
+        """
+        self.con = con
+        self.file_id = file_id
+
+    def upload_part(self, part_num, data):
+        """
+        Send part of file for multipart method.
+
+        :param part_num: integer matching part number.
+        :param data: File object or a data chunk to upload
+        """
+        self.con.raw.post(
+            '/files/multipart/upload',
+            params=dict(file_id=self.file_id, part_num=part_num), files=dict(file=data)
         )
 
-        return data['file_id']
+    def complete(self):
+        """
+        Finish upload process using multipart method. Returns the Shooju file id.
+
+        :return: str: Uploaded file id
+        """
+        self.con.raw.post(
+            '/files/multipart/complete',
+            params=dict(file_id=self.file_id)
+        )
+        return self.file_id
+
 
 class GetBulk(object):
     """
